#!/usr/bin/perl -w

# differet fro all_seq_gr_info.pl in that file motif is a paramater

# Prg: all_seq_gr_info.pl

# INPUT: 	xxx_nr.fas fasta file with only non_redundant sequences in it
#		xxx_redundant.tbl (output of group_seq_var1.pl; [0] redundant seq code [1] longest identical seq code)
#		xxx_multihit_final.tbl (output of group_seq_var1.pl; hit lines from blast tbl)
#		xxx_gr.txt (output of calc_indent1.pl; hit lines where overhang is to long)
#		xxx_cl.txt (output of calc_indent1.pl; )

#only 1 series of files in the input folder at a time 

# OUTPUT: xxx_all_info.txt
#		[0] SEQ
#		[1] IDEM list of code of sequences that were redundants 100% ident on the whole overlaping region
#		[2] FLANK_IDEM sequences whre flanking region is idem, but MS is diff
#		[3] FLANK95 sequences with flankig region sim over predefined limit
#		[4] GR sequences similar but identity is lower, that predefined limit
#		[5] MULTIHIT sequenes producing more that one hit with the given seq.
# 		[6] category: 1 no hit; 2 red, 3 ms alleles 4 flank allele 5 gr (multihit not included) 
#			if multiple category only the highest number is indicated 

  use warnings;
  use strict;
  use subprogramEM;

my $folder = 'datatemp1';
my $outfolder = 'temp1';
my $gr_limit = 0.05;
my $file_mot = '';

if (scalar@ARGV == 4)
{
	$folder = $ARGV[0];
	$outfolder = $ARGV[1];
	$gr_limit = $ARGV[2];
	$file_mot = $ARGV[3];
}
else
{
	$folder = 'datatemp';
	$outfolder = 'temp';
	$gr_limit = 0.05;
	$file_mot = '_nr.fas';
}

  unless ( opendir(FOLDER, $folder) )
  {
      print "Cannot access to folder $folder\n";
      exit;
  }
my @filenames = grep ( !/^\.\.?$/, readdir(FOLDER) );
closedir(FOLDER);

# if diffrenece between two sequences is less than $gr_limit sequences are cosidered as alleles 
my $fas_file = '';
my $gr_file = '';
my $cl_file = '';
my $multihit_file = '';
my $red_file = '';
my $outfile = ''; 

# define filenames
foreach my $filename (@filenames)
{
	if ($filename =~ /$file_mot$/)
	{
		$outfile = $filename;
		$outfile =~ s/\.fas/_all_info\.txt/;
		$outfile = $outfolder.'/'.$outfile;
	$filename = $folder.'/'.$filename;
		$fas_file = $filename;
		$gr_file = $fas_file;
		$gr_file =~ s/_nr\.fas/_gr\.txt/;
		$cl_file = $fas_file;
		$cl_file =~ s/_nr\.fas/_cl\.txt/;
		$multihit_file = $fas_file;
		$multihit_file =~ s/_nr\.fas/_multihit_final\.tbl/;
		$red_file = $fas_file;
		$red_file =~ s/_nr\.fas/_redundant\.tbl/;


my %red = open_file_and_hach($red_file, '1'); 

# determine all seq code that are idem to the same long seq
my %red_gr = group_red(\%red);
eliminate_last_comma_in_hach(\%red_gr, ',');

my %multihit = group_hit_in_blast_tbl($multihit_file);
eliminate_last_comma_in_hach(\%multihit, ',');

my %gr = group_hit_in_blast_tbl($gr_file);

my %flank_idem = ();
my %flank_95 = ();
sort_out_clust_res($cl_file, \%flank_idem, \%flank_95, \%gr, $gr_limit);
eliminate_last_comma_in_hach(\%flank_idem, ',');
eliminate_last_comma_in_hach(\%flank_95, ',');
eliminate_last_comma_in_hach(\%gr, ',');

#print_hachage(\%multihit);

open_file_out($outfile, 'OUT');
print OUT "SEQ;IDEM;FLANK_IDEM;FLANK95;GR;MULTIHIT;CATEGORY\n";
open_file_in($fas_file, 'IN');
# put a tegory into last colomn: 1; no hit at all; 2 onle reduntdans (100% ident all along); 3 ms allele (100% ident in flanking, but diff in ms)
# 4 flanking allele (flanking ident. is above limit (95%)), 5 group (flanking id is under ident. limit)
# if more than one category applies to a seq. only the higest number is indicated (e.g. if seq has redundants, 
# no ms alleles have flank alleles, but no group, the ctegory is 4); ATTENTION!!! the multihit is not treated here  
while (my $seq = <IN>)
{
my $cat = 1;
	if ($seq =~ />.+/ )
	{
		my $code = $&;
		$code =~ s/>//;
		print OUT "$code;";
		if (exists $red_gr{$code})
		{
			print OUT "$red_gr{$code};";
			$cat = 2;
		} 
		else
		{print OUT ";";}

		if (exists $flank_idem{$code})
		{
			print OUT "$flank_idem{$code};";
			$cat = 3;
		} 
		else
		{print OUT ";";}

		if (exists $flank_95{$code})
		{
			print OUT "$flank_95{$code};";
			$cat = 4;
		} 
		else
		{print OUT ";";}

		if (exists $gr{$code})
		{
			print OUT "$gr{$code};";
			$cat = 5;
		} 
		else
		{print OUT ";";}

		if (exists $multihit{$code})
		{
			print OUT "$multihit{$code};";
		} 
		else
		{print OUT ";";}
	
		print OUT "$cat;\n";

	}
}
close (IN);
close (OUT);

# end if file_mot
	}
# end foreach filename
}

 exit;

###########################################################

sub group_red
{
my ($red) = @_;
my %red_gr = ();

foreach my $code (keys %$red)
{
	$red_gr{$$red{$code}} .= $code.',';
}

return %red_gr;
}
###########################################################

sub group_hit_in_blast_tbl
{
my ($multihit_file) = @_;
my %multihit = ();
open_file_in($multihit_file, 'IN');

while (my $line = <IN>)
{
	my @line = split(' ', $line);
	unless (exists $multihit{$line[0]})
	{
		$multihit{$line[0]} = '';
	}
	unless (exists $multihit{$line[1]})
	{
		$multihit{$line[1]} = '';
	}
	unless ($multihit{$line[0]} =~ /$line[1],/)
	{
		$multihit{$line[0]} .= $line[1].',';
	}
	unless ($multihit{$line[1]} =~ /$line[0],/)
	{
		$multihit{$line[1]} .= $line[0].',';
	}
}
close (IN);


return %multihit;
}
###########################################################
sub sort_out_clust_res
{
my ($cl_file, $flank_idem, $flank_95, $gr, $gr_limit) = @_;

open_file_in($cl_file, 'IN');

while (my $line = <IN>)
{
	my @line = split(';', $line);
	if ($line[11] == 0)
	{
		$$flank_idem{$line[0]} .= $line[1].',';
		$$flank_idem{$line[1]} .= $line[0].',';
	} 
	elsif ($line[11] < $gr_limit)
	{
		$$flank_95{$line[0]} .= $line[1].',';
		$$flank_95{$line[1]} .= $line[0].',';		
	}
	else
	{
		$$gr{$line[0]} .= $line[1].',';
		$$gr{$line[1]} .= $line[0].',';		
	}
}
close (IN);

}
###########################################################

sub eliminate_last_comma_in_hach
{
my ($hach, $char) = @_;
foreach my $code (keys %$hach)
{
	$$hach{$code} =~ s/$char$//;
}
}
