#!/usr/bin/perl -w

# name of the program: sort_tag_beg.pl

# reads all, sequences from a fasta file and identify which of the given tag is present, removes tag (and adaptor)
# writes one fasta file per tag with all sequences that had the given tag.
# if blast hit is shorter then adaptor, removes the adaptor length not the length of the hit

# INPUT: 	xxx.tag (fasta file with vectors)
# 		yyy.fas fasta file to be cleaned and sorted out.

# OUPUT: yyy_m8.tbl results of blast againts te adapter file
#		yyy_wov.fas 1 fasta file with all of the cleaned sequences that had the given adapter (tag)


  use warnings;
  use strict;
  use subprogramEM;

my $syst = '';
my $folder = 'datatemp1';
my $outfolder = 'temp1';
my $blast_path = 'c:/BLAST2_2_18/bin/';
my $Evector = '1E-3';
my $tag_file = '';
my $motif = '';
# if $noadapt ==1 writes one fasta file with sequences that had adapter at the begining, and another if they did not. 
my $noadapt = 1;

if ( scalar@ARGV == 7 )
{
   	$syst = $ARGV[0];
   	$folder = $ARGV[1];
   	$outfolder = $ARGV[2];
   	$blast_path = $ARGV[3];
   	$Evector = $ARGV[4];
   	$tag_file = $ARGV[5];
	$motif = $ARGV[6];
}
else
{
	$syst = 'win';
	$folder = 'datatemp1';
	$outfolder = 'temp1';
	$blast_path = 'c:/BLAST2_2_18/bin/';
	$Evector = '1E-3';
	$tag_file = 'd:\prg454\454data\1.cut_vector_brut\adapter.fas';
	$motif = '\.tag';
}

my $fas_file = '';
# if $dist == 1 produce length distridution data
my $dist = 0;

  unless ( opendir(FOLDER, $folder) )
  {
      print "Cannot access to folder $folder\n";
      exit;
  }
my @filenames = grep ( !/^\.\.?$/, readdir(FOLDER) );
closedir(FOLDER);

# identify files
foreach my $fas_file (@filenames)
{
if ($fas_file =~ /$motif$/)
{
my %bl_beg = ();
my %bl_end = ();
my %seq = ();
my %adapt = ();


	my $outfile = $fas_file;
	$outfile =~ s/$motif$/\.wov/;
	$outfile = $outfolder.'/'.$outfile;
	my $out_blast = $fas_file;
	$out_blast =~ s/$motif$/_m8\.tbl/;
	$out_blast = $outfolder.'/'.$out_blast;
	my $out_length = $fas_file;
	$out_length =~ s/$motif$/_length\.tbl/;
	$out_length = $outfolder.'/'.$out_length;

	$fas_file = $folder.'/'.$fas_file;


# blast sequence file againts the adaptor file; filter is switched off, *
#to be able to clean even low complexity sequences
call_blast_adapter_no_filter($fas_file, $out_blast, $tag_file, $Evector, $blast_path);
my $fold = $tag_file;
$tag_file =~ /[a-z0-9_\.]+$/i;
my $t = $&;
$fold =~ s/$t//;
$fold =~ s/\\$//;
$fold =~ s/\/$//;
#print "$fold\n";
if ($syst eq 'win')
{
	delete_blast_temp_files_win1($fold);
}
elsif ($syst eq 'linux')
{
	delete_blast_temp_files($fold);
}

%seq = read_hach_from_fasta_code_wo_space($fas_file);
 my %seq_l = length_sequences(\%seq);
#%adapt = read_hach_from_fasta_code_wo_space($tag_file);
#my %adapt_l = length_sequences(\%adapt);


open_file_in($out_blast, 'BL');
while(my $line = <BL>)
{
	my $code = '';
	my @line = split(' ', $line);
	if ($line[6] < $seq_l{$line[0]} - $line[7])
	{
		if (exists $bl_beg{$line[0]})
		{
			if ($line[7]> $bl_beg{$line[0]})
			{
				$bl_beg{$line[0]} = $line[7];
			}
		}
		else
		{
			$bl_beg{$line[0]} = $line[7];
		}
	}
	else
	{
		if (exists $bl_end{$line[0]})
		{
			if ($line[6] < $bl_end{$line[0]})
			{
				$bl_end{$line[0]} = $line[6];
			}
		}
		else
		{
			$bl_end{$line[0]} = $line[6];
		}
	}
	
}
close BL;

#print_hachage(\%bl_end);
foreach my $end (keys %bl_end)
{
	my $seq_t = substr($seq{$end}, 0, $bl_end{$end}-1);
	$seq{$end} = $seq_t;
#	print FAS ">$end\n$seq_t\n";
}
foreach my $beg (keys %bl_beg)
{
	my $l = length $seq{$beg};
	if ($l - $bl_beg{$beg}> 0)
	{
		my $seq_t = substr($seq{$beg}, $bl_beg{$beg}, $l - $bl_beg{$beg});
		$seq{$beg} = $seq_t;
#		print FAS ">$beg\n$seq_t\n";
	}
	else
	{
		$seq{$beg} = '';
	}
}

my $out_noadapt = $outfile;
$out_noadapt =~ s/\.wov/\.woa/;
if ($noadapt ==1)
{
	open_file_out($out_noadapt, 'NOA');
}
open_file_out($outfile, 'FAS');
open_file_out($out_length, 'OUT');
print OUT "code;orig_length;bases_cut_beg;bases_cut_end;length_wov;\n";
my %orig = ();
my %cut = ();
my %diff =();
my $step = 10;

foreach my $seq (sort keys %seq)
{
	my $int = int($seq_l{$seq}/$step)*$step;
	++$orig{$int};
	if (exists $bl_beg{$seq})
	{
		print FAS ">$seq";
		print FAS "_A\n$seq{$seq}\n";
		print OUT "$seq";
		print OUT "_A;$seq_l{$seq};";
		print OUT "$bl_beg{$seq};";
	}
	elsif ($noadapt ==1)
	{
		print NOA ">$seq";
		print NOA "_N\n$seq{$seq}\n";
		print OUT "$seq";
		print OUT "_N;$seq_l{$seq};";
		print OUT "0;";
	}
	else
	{
		print FAS ">$seq";
		print FAS "_N\n$seq{$seq}\n";
		print OUT "$seq";
		print OUT "_N;$seq_l{$seq};";
		print OUT "0;";		
	}

	if (exists $bl_end{$seq})
	{
		my $t = $seq_l{$seq} - $bl_end{$seq} +1;
		print OUT "$t;";
	}
	else
	{
		print OUT "0;";
	}
	my $l = length $seq{$seq};
	$int = int($l/$step)*$step;
	++$cut{$int};
	$int = int(($seq_l{$seq}-$l)/$step)*$step;
	++$diff{$int};
	print OUT "$l;\n";

}
close OUT;
close FAS; 	
if ($noadapt == 1)
{
	close NOA;
}

if ($dist == 1)
{
#open_file_out($out_dist, 'DIS');
#print DIS "interval;number of seq of which the original length is in interval;numb_cut;orig-cut;\n";

for(my $i = 0; $i < 610; $i = $i+$step)
{
#	print DIS "$i;";
	if (exists $orig{$i})
	{
#		print DIS "$orig{$i};";
	}
	else
	{
#		print DIS "0;";
	}
	if (exists $cut{$i})
	{
#		print DIS "$cut{$i};";
	}
	else
	{
#		print DIS "0;";
	}
	if (exists $diff{$i})
	{
#		print DIS "$diff{$i};";
	}
	else
	{
#		print DIS "0;";
	}

#	print DIS "\n";

}
#close DIS;
#print_hachage(\%cut);
}


my $motdel = '';
if ($syst eq 'win')
{
	$motdel = '_m8.tbl';
	delete_files_motif_win($outfolder, $motdel);
}
elsif ($syst eq 'linux')
{
	$motdel = '_m8.tbl';
	delete_files_motif($outfolder, $motdel);
}

# end of if motif
}
#end of foreach fas_file
}
exit;

