#!/usr/bin/perl -w

# Nom du programme:primer3_input_lc.pl 

#INPUT: xxx_selectxxxxx.fas (output file of select_best_ms_info_in_def_line.pl, target ms motifs are 
# printed in lc in definition line )

# makes input file for primer3 with excludee and target regions. 
#If more than 1 target ms is present in a given seq. writes the sequence as many times as target ms. 
#Gives the codes of the sequences that are present more than once in the file
# gives a summary stat  the target ms: sequence, xth target ms of a given seq, firs pos, legth bp, length rep, motif, motif_trans

#OUTPUT:
# xxx_selectxxxxx_primer3.txt (input file for primer3)
# xxx_selectxxxxx_target_ms_stat.tbl (MS_ID;FIRST_POS;LENGTH_BP;LENGTH_REP;MOTIF;TRANS_MOTIF;)





  use warnings;
  use strict;
  use subprogramEM;


my $folder = '';
my $outfolder = '';
my $min_rep = '';
my $ampl_min = '';
my $flank_min = '';
my $fas_motif = '';

if (scalar@ARGV == 6)
{
	$folder = $ARGV[0];
	$outfolder = $ARGV[1];
	$min_rep = $ARGV[2];
	$ampl_min = $ARGV[3];
	$flank_min = $ARGV[4];
	$fas_motif = $ARGV[5];
}
else
{
	$folder = 'datatemp1';
	$outfolder = 'temp1';
	$min_rep = 5;
	$ampl_min = 100;
	$flank_min = 30;
	$fas_motif = 'fas';
}

my $trans_file = 'motifs_transmot_hach1_6.txt';
my %trans = open_file_and_hach($trans_file, '1');

# make lc motifs in %trans
foreach my $mot (keys %trans)
{
	my $mot_lc = lc $mot;
	$trans{$mot_lc} = $trans{$mot};
}


  unless ( opendir(FOLDER, $folder) )
  {
      print "Cannot access to folder $folder\n";
      exit;
  }
my @filenames = grep ( !/^\.\.?$/, readdir(FOLDER) );
closedir(FOLDER);

foreach my $file (@filenames)
{
if ($file =~ /$fas_motif$/)
{
	my $out_primer = $file;
	$out_primer =~ s/\.fas/_primer3\.txt/;
	$out_primer = $outfolder.'/'.$out_primer;
	my $out_stat = $out_primer;
	$out_stat =~ s/_primer3\.txt/_target_ms_stat\.tbl/;
    $file = $folder.'/'.$file;

	open_file_out($out_stat, 'STAT');
	open_file_out($out_primer, 'PR');
	if ($fas_motif =~ /pool/i)
	{
		print STAT "MS_ID;FIRST_POS;LENGTH_BP;LENGTH_REP;MS;\n";
	}	
	else
	{
		print STAT "MS_ID;FIRST_POS;LENGTH_BP;LENGTH_REP;MOTIF;TRANS_MOTIF;\n";
	}
	open_file_in($file, 'DATA');

while (my $seq = one_seq($file))
{
	if ($seq eq '>')
	{
		next;
	}
	$seq =~ /.*?\n/;
	my $def = $&;
	$seq =~ s/$def//;
	chomp $def;
	$def =~ /.+?\s/;
	my $code = $&;
	$def =~ s/$code//;
	$code =~ s/\s//;
	my @ms = split(';', $def);

	$seq =~ s/\s//g;
	$seq =~ s/>//;
	$seq = uc $seq;
#	print "$seq***********\n";
	my $ms_id = 0;
	my $seq_l = length $seq;

#If more than 20% of the sequnece is N ignore the sequence (probably bad quality concensus sequence) 
	my $N = 0;
	while ($seq =~ /N/ig)
	{
		++$N;
	}
	if ($N/(length$seq) > 0.2)
	{
		next;
	}

	
	for (my $i = 0; $i< scalar@ms; $i = $i+4)
	{
		my $beg = 0;
		my $end = $seq_l;
# target microsatellit motifs are printed in lower case (p is for pool) 
		if ($ms[$i] =~ /[actgp]/)
		{
			print PR "PRIMER_SEQUENCE_ID=$code";
			my $ms_l = $ms[$i+2] - $ms[$i+1] +1;
			print PR "_$ms_id\nSEQUENCE=$seq\nTARGET=$ms[$i+1],$ms_l\n";

			my $excl = '';
			unless ($i == 0)
			{
				$beg = $ms[$i-2];
			}
			unless ($i +4 == scalar@ms)
			{
				$end = $ms[$i+5];
			}
			unless ($beg == 0)
			{
				$excl .= '1,'.$beg;
			}
			unless ($end == $seq_l)
			{
				my $end_l = $seq_l - $end;
				$excl .= ' '.$end.','.$end_l;
			}
			$excl =~ s/^\s//;
			if ($excl eq '')
			{
				print PR "=\n";
			}
			else
			{
				print PR "EXCLUDED_REGION=$excl\n=\n";
			}
			
			if ($ms[$i] eq 'pool')
			{
				my $l = $ms[$i+2] - $ms[$i+1] +1;
				my $pool_mot = substr($seq, $ms[$i+1]-1, $l);
				print STAT "$code";
				print STAT "_$ms_id;$ms[$i+1];$ms_l;$ms[$i+3];$pool_mot;\n";
				++$ms_id;
			}
			else
			{	
				print STAT "$code";
				print STAT "_$ms_id;$ms[$i+1];$ms_l;$ms[$i+3];$ms[$i];$trans{$ms[$i]};\n";
				++$ms_id;
			}

# end if
		}
# end for
	}
}
	close DATA;
	close STAT;
	close PR;
#end if fas_mot
}
# end foreach file
}



exit;

############################################################


sub one_seq
{
#    read the data concernant one sequence

    my ($filename) = @_; 
    my ($res_seq) = ''; 

    my ($sep_entree) = $/;
    $/ = ">";
    $res_seq = <DATA>;
    $/ = $sep_entree;
    return $res_seq;
}


##########################################################

