#!/usr/bin/perl -w

# Nom du programme:select_best_msinfo_in_def_line2.pl 
# Modified from select_best_msinfo_in_def_line.pl => file_mot is a parameter

#INPUT: xxx_select.seq (output file of exatract_perfect_microsat.pl or find and mask_ms.pl)
# [0] SEQ_CODE
# [1] NUMBER OF MS
# [2] SEQ_LENGTH
# [3] MS MOTIF
# [4] MS FIRTS POS
# [5] MS LAST POS
# [6] NUMBER OF REP
# [3-6] REPEATED FOR ALL MS
#       xxx_select.fas 
#       ONLY 2 FILES IN FOLDER

# select sequences that have at least X bp flankig region, amplicon at least Y bp, does not contain mico-ms
# $flank_min = 30;
# $ampl_min = 100;    minimum length for amplicon 
# $mono_max = 4;      max number of consecutive repeats of monobase in the flanking region
# $non_mono_max = 2;  max number of consecutive repeats of di-hexa motif in the flanking region
#$min_mot_l = 2;      min length of target ms motif 
#$max_mot_l = 6;      max length of target ms motif 
#$min_rep = 6;        minimun repeat number for target ms  




#OUTPUT:
# xxx_12345.fas (fasta file with selected sequences; all nuisible (including target) ms are printed 
# in lower case letters) that five numbers in the filename correspond to $mono_max $non_mono_max 
#$min_mot_l $max_mot_l $min_rep; previous lower case masks are deleted
# xxx_non_select.seq (seq file with only data on non selected sequences; keeps all info on line in original seq file)




  use warnings;
  use strict;
  use subprogramEM;


my $folder = '';
my $outfolder = '';
my $flank_min = 0;
# minimum length for amplicon
my $ampl_min = 0;
# max number of consecutive repeats of monobase in the flanking region
my $mono_max = 0;
#  max number of consecutive repeats of di-hexa motif in the flanking region
my $non_mono_max = 0;
#min length of target ms motif 
my $min_mot_l = 0;
#max length of target ms motif 
my $max_mot_l = 0;
#minimun repeat number for target ms 
my $min_rep = 0;
my $file_mot = '';

if (scalar@ARGV == 10)
{
	$folder = $ARGV[0];
	$outfolder = $ARGV[1];
	$flank_min = $ARGV[2];
	$ampl_min = $ARGV[3];
	$mono_max = $ARGV[4];
	$non_mono_max = $ARGV[5];
	$min_mot_l = $ARGV[6];
	$max_mot_l = $ARGV[7];
	$min_rep = $ARGV[8];
	$file_mot = $ARGV[9];

}
else
{
	$folder = 'datatemp1';
	$outfolder = 'temp1';
	$flank_min = 20;
# minimum length for amplicon
	$ampl_min = 100;
# max number of consecutive repeats of monobase in the flanking region
	$mono_max = 4;
#  max number of consecutive repeats of di-hexa motif in the flanking region
	$non_mono_max = 2;
#min length of target ms motif 
	$min_mot_l = 2;
#max length of target ms motif 
	$max_mot_l = 6;
#minimun repeat number for target ms 
	$min_rep = 5;
	$file_mot = '\.seq';
}

  unless ( opendir(FOLDER, $folder) )
  {
      print "Cannot access to folder $folder\n";
      exit;
  }
my @filenames = grep ( !/^\.\.?$/, readdir(FOLDER) );
closedir(FOLDER);

my $seq_file = '';
my $fas_file = '';
foreach my $file (@filenames)
{
if ($file =~ /$file_mot$/)
{
	my $select_seq = $file;
	$select_seq =~ s/\./_select$flank_min$ampl_min$min_rep\./;
	$select_seq = $outfolder.'/'.$select_seq;

    $file = $folder.'/'.$file;
	$seq_file = $file;
	$fas_file = $file;
	$fas_file =~ s/\.seq/\.fas/;

my $select_fas = $select_seq;
$select_fas =~ s/\.seq/\.fas/;
my $non_select_seq = $select_seq;
$non_select_seq =~ s/_select$flank_min$ampl_min$min_rep/_NONselect$flank_min$ampl_min$min_rep/;

my %good_target = ();

open_file_in($seq_file, 'SEQ');
open_file_out($select_fas, 'FAS');
open_file_out($non_select_seq, 'NSEL');
my %seq = read_hach_from_fasta_code_wo_space($fas_file);
my $c = 0;

my $title = <SEQ>;
while (my $line = <SEQ>)
{
#	print "$line";
    my @line = split(';', $line);
# to avoid last empty element after the last ;
    pop@line;
if ($line[2]> $ampl_min -1)
{
    my %pos = ();
    my %target = ();
	my %pos_motif = ();
	my %pos_rep = ();
    for (my $i = 3; $i <scalar@line; $i=$i+4)
    {
# select target ms
	if (($min_mot_l -1 < length $line[$i]) and  (length $line[$i] < $max_mot_l+1) and ($line[$i+3] > $min_rep-1) and ($line[$i+1] > $flank_min) and (($line[2]-$line[$i+2]) >$flank_min-1))
	{
	    $target{$line[$i+1]} = $line[$i+2];
	    $pos{$line[$i+1]} = $line[$i+2];
	    $pos_motif{$line[$i+1]} = $line[$i];	
	    $pos_rep{$line[$i+1]} = $line[$i+3];
	}
# nuisibles ms if in flanking region	
	else
	{
# N(s)
#		if ($line[$i] eq 'N')
#		{
#			$pos{$line[$i+1]} = $line[$i+2];
#		      $pos_motif{$line[$i+1]} = $line[$i];	
#		}
#mono_base ms 
	    if (length $line[$i] == 1 and $line[$i+3]> ($mono_max))
	    {
		$pos{$line[$i+1]} = $line[$i+2];
	      $pos_motif{$line[$i+1]} = $line[$i];
	      $pos_rep{$line[$i+1]} = $line[$i+3];	
	    }
#di_hexa ms
	    if (length $line[$i] >1 and $line[$i+3]> ($non_mono_max) )
	    {
		$pos{$line[$i+1]} = $line[$i+2];
 	      $pos_motif{$line[$i+1]} = $line[$i];	
		$pos_rep{$line[$i+1]} = $line[$i+3];
	    }
	}
    }
#print "$line";
#    print_hachage(\%pos);
#    print "target\n";
#    print_hachage(\%target);

# check for each putative target ms if fkanking region without nuisible is long enough

    my @pos = sort {$a <=> $b}keys %pos;
    my @target = sort {$a <=> $b}keys %target;
    my $good_target = 0;
# hach that contains the position of the good target ms in @pos
	my %good_target_pos = ();

    if (scalar @target >0)
    {
	foreach my $target (@target)
	{
# determine the position of the target ms in the table of @pos (nuisible ms in flanking)
	    my $target_in_table = 0;
	    for (my $i = 0; $i < scalar @pos; ++$i)
	    {
		if ($pos[$i] == $target)
		{
		    $target_in_table = $i;
		    $i = scalar @pos;
		}
	    }

# if target ms is the first and last of nuisible list (only ms)
	    if (scalar@pos ==1 )
	    {
		$good_target = 1;
		$good_target_pos{$target} = '';
	    }
# if target ms is first in nuisible list
	    elsif ($target_in_table == 0)
	    {
		if ( ($pos[$target_in_table+1]-$target{$target} > $flank_min) and ($pos[$target_in_table+1] > $ampl_min))
		{
		    $good_target = 1;
		  	$good_target_pos{$target} = '';
		}
	    }

# if target ms is the last in nuisible list
	    elsif ($target_in_table == scalar@pos -1)
	    {
#		if (($target - $pos{$pos[$target_in_table-1]} > $flank_min) and (length $line[2] - $pos{$pos[$target_in_table-1]} > $ampl_min) )
		if (($target - $pos{$pos[$target_in_table-1]} > $flank_min) and ($line[2] - $pos{$pos[$target_in_table-1]} > $ampl_min-1) )
		{
#		print "$line[0]\n";
		    $good_target = 1;
		$good_target_pos{$target} = '';
		}
	    }
#check if distance between target ms and nearests nuisible ms if longer than $flank_min and if the amplicon is at least $ampl_min long
# $target - $pos{$pos[$target_in_table-1]}   first pos of target ms - last pos of prev nuisible ms
# $pos[$target_in_table+1]-$target{$target} first pos of first nuisible ms followig the target ms - last pos of target ms
	    elsif (($target - $pos{$pos[$target_in_table-1]} > $flank_min) and ($pos[$target_in_table+1]-$target{$target} > $flank_min) and ($pos[$target_in_table+1]- $pos{$pos[$target_in_table-1]} > $ampl_min))
	    {
		$good_target = 1;
		$good_target_pos{$target} = '';
	    }
	    
#end foreach	
	}
#end if
    }

if ($good_target == 1)
{
#	print SEL "$line[0];";
	print FAS ">$line[0]";
	if ($line[0] =~ /concat/)
	{
		print FAS "_A ";
	}
	else
	{
		print FAS " ";
	}
    foreach my $pos (@pos)
    {
	if (exists $good_target_pos{$pos})
	{
		$pos_motif{$pos} = lc $pos_motif{$pos};
		++$c;
	}
#	print SEL "$pos_motif{$pos};$pos;$pos{$pos};";	
	print FAS "$pos_motif{$pos};$pos;$pos{$pos};$pos_rep{$pos};";	

    }
	$seq{$line[0]} = uc $seq{$line[0]};
	my $fragment = change_ms_into_lower_case($seq{$line[0]}, \%pos);
#	print SEL "\n$seq{$line[0]}\n";
	print FAS "\n$fragment\n";
#	print SEL "\n";

}
else
{
	print NSEL "$line";
}
#end if ampl_min
}
#end while (1 line)
}
print "Number of target ms: $c\n";
close SEQ;
close FAS;
#close SEL;
close NSEL;

# end if file_mot
}
#end foreach filename
}

exit;


