#!/usr/bin/perl -w

# name of the program: sort_tag_beg.pl

# reads all, sequences from a fasta file and identify which of the given tag is present, removes tag (and adaptor)
# writes one fasta file per tag with all sequences that had the given tag.
# if blast hit is shorter then adaptor, removes the adaptor length not the length of the hit

# INPUT: 	tag.fas (fasta file with only the possible tags in it
# 		yyy.fas fasta file to be cleaned and sorted out.

# OUPUT: yyy_m8.tbl results of blast againts te adapter file
#		yyy_adX.fas 1 fasta file with all of the cleaned sequences that had the given adapter (tag)
#		yyy_not_cleaned.fas file with sequences where adapters wre not removed
#		yyy_m8_rest.tbl results of blast againts te adapter file that are not 'treated'


  use warnings;
  use strict;
  use subprogramEM;

my $syst = '';
my $folder = '';
my $outfolder = '';
my $Evector = '';
my $word_size = '';
my $blast_path = '';
my $tag_file = '';

  	
if ( scalar@ARGV == 7 )
{
   	$syst = $ARGV[0];
   	$folder = $ARGV[1];
   	$outfolder = $ARGV[2];
   	$blast_path = $ARGV[3];
   	$Evector = $ARGV[4];
   	$word_size = $ARGV[5];
   	$tag_file = $ARGV[6];
}
else
{
	$syst = 'win';
	$folder = 'datatemp1';
	$outfolder = 'temp1';
	$Evector = '1E-1';
	$word_size = 7;
	$blast_path = 'c:/BLAST2_2_18/bin/';
   	$tag_file = 'd:\prg454\454data\1a_sort_tag_beg_orig\tag.fas';
}



my $fas_file = '';
  unless ( opendir(FOLDER, $folder) )
  {
      print "Cannot access to folder $folder\n";
      exit;
  }
my @filenames = grep ( !/^\.\.?$/, readdir(FOLDER) );
closedir(FOLDER);

# identify files
foreach my $fas_file (@filenames)
{
my %adapt = ();
my %seq = ();
my %sort = ();


	my $outfile = $fas_file;
	$outfile =~ s/\.fas/_NOTAG\.tag/;
	$outfile = $outfolder.'/'.$outfile;
	my $out_blast = $fas_file;
	$out_blast =~ s/\.fas/_m8\.tbl/;
	$out_blast = $outfolder.'/'.$out_blast;
	my $blast_rest = $fas_file;
	$blast_rest =~ s/\.fas/_m8_rest\.tbl/;
	$blast_rest = $outfolder.'/'.$blast_rest;
	my $blast_good = $fas_file;
	$blast_good =~ s/\.fas/_m8_good\.tbl/;
	$blast_good = $outfolder.'/'.$blast_good;

	$fas_file = $folder.'/'.$fas_file;



# blast sequence file againts the adaptor file; filter is switched off, *
#to be able to clea even low complexity sequences
call_blast_adapter_no_filter_word_size($fas_file, $out_blast, $tag_file, $Evector, $blast_path, $word_size);
my $fold = $tag_file;
$tag_file =~ /[a-z0-9_\.]+$/i;
my $t = $&;
$fold =~ s/$t//;
$fold =~ s/\\$//;
$fold =~ s/\/$//;
#print "$fold\n";
if ($syst eq 'win')
{
	delete_blast_temp_files_win1($fold);
}
elsif ($syst eq 'linux')
{
	delete_blast_temp_files($fold);
}

%seq = read_hach_from_fasta_code_wo_space($fas_file);
 my %seq_l = length_sequences(\%seq);
%adapt = read_hach_from_fasta_code_wo_space($tag_file);
my %adapt_l = length_sequences(\%adapt);


open_file_in($out_blast, 'BL');
#open_file_out($blast_rest, 'OBL');
#open_file_out($blast_good, 'GBL');

while(my $line = <BL>)
{
	my @line = split(' ', $line);
# hit sur brin +, hit at the begining of the sequence (+2 for to allow for 1 seq error (indel))
	if ($line[8] < $line[9] and $line[7] < $adapt_l{$line[1]} +2)
	{
		$sort{$line[1]} .= $line[0].';';
		if ($line[7]> $adapt_l{$line[1]})
		{
#			print "$seq{$line[0]}\n";
			$seq{$line[0]} = substr($seq{$line[0]}, $line[7], $seq_l{$line[0]} - $line[7]);
#			print "$seq{$line[0]}\n";
#			print GBL "$line";

		}
		else
		{
#			print "$line[0]\n$seq{$line[0]}\n";
			$seq{$line[0]} = substr($seq{$line[0]}, $adapt_l{$line[1]}, $seq_l{$line[0]} - $adapt_l{$line[1]});	
#			print "$seq{$line[0]}\n";
#			print GBL "$line";
		}
	}
	else
	{
#		print OBL "$line";
	}
}

close (BL);
#close (OBL);


#print_hachage(\%sort);

foreach my $adapt (keys %sort)
{
	my @list = split(';', $sort{$adapt});
	$fas_file =~ /[a-z0-9_\.]*$/i;
	my $tempfile = $&;
	my $out = $tempfile;
	$out =~ s/\.fas/_$adapt\.tag/;
	$out = $outfolder.'/'.$out;
	open_file_out($out, 'OUT');
	my $i = 0;
	foreach my $code (@list)
	{
		if ($seq{$code} eq '')
		{
			print "In $code an adapter is already removed\n";
		}
		else
		{
			print OUT ">$code";
			print OUT "_$adapt\n$seq{$code}\n";
			$seq{$code} = '';
			++$i;
		}
	}
	close OUT;
	print "Number of sequences with $adapt tag: $i\n";
}
open_file_out($outfile, 'OUT');

# prints a file with the non_cleaned sequences
my $i = 0;
foreach my $code (keys %seq)
{
	unless ($seq{$code} eq '')
	{
		print OUT ">$code\n$seq{$code}\n";
		++$i;
	}
}
	print "Number of sequences without tag: $i\n";

close (OUT);

my $motdel = '';
if ($syst eq 'win')
{
	$motdel = '_m8.tbl';
	delete_files_motif_win($outfolder, $motdel);
}
elsif ($syst eq 'linux')
{
	$motdel = '_m8.tbl';
	delete_files_motif($outfolder, $motdel);
}
#end of each file
}

exit;

